==> allensdk-1.3.2-py3-none-any.whl: allensdk/brain_observatory/behavior/swdb/save_trial_response_df.py <==
import sys
import os
import numpy as np
import pandas as pd
from scipy import stats
import itertools

from allensdk.brain_observatory.behavior.behavior_ophys_session import BehaviorOphysSession
from allensdk.brain_observatory.behavior.behavior_ophys_api.behavior_ophys_nwb_api import BehaviorOphysNwbApi
from allensdk.internal.api.behavior_ophys_api import BehaviorOphysLimsApi

from allensdk.brain_observatory.behavior.swdb import behavior_project_cache as bpc
from importlib import reload; reload(bpc)
from allensdk.brain_observatory.behavior.swdb.analysis_tools import get_nearest_frame, get_trace_around_timepoint, get_mean_in_window

'''
    This file contains functions and a script for computing the trial_response_df dataframe.
    This file was hastily constructed before friday harbor. Places where there are known issues are flagged with PROBLEM
'''


def add_p_vals_tr(tr,response_window = [4,4.5]):
    '''
        Computes the p value for each cell's response on each trial. The p-value is computed using the function 'get_p_val' 

        INPUT:
        tr, trial_response_dataframe
        response_window, the time points in the dff trace to use for computing the p-value. 
                        PROBLEM: The default value here assumes that the dff_trace starts 4 seconds before the change time. 
                        This should be set up with more care and flexibility. 

        OUTPUTS:
        tr, the same trial_response_dataframe, with a new column 'p_value' appended. 
    
        ASSERTS:
        tr['p_value'] is inclusively bounded between 0 and 1, and does not include NaNs
    '''
    
    # Set up empty column
    tr['p_value'] = 1.
    ophys_frame_rate=31. #Shouldn't hard code this PROBLEM
    
    # Iterate over trial/cell pairs, and compute p-value
    for index,row in tr.iterrows():
        tr.at[index,'p_value'] = get_p_val(row.dff_trace, response_window, ophys_frame_rate)

    # Test to ensure p values are bounded between 0 and 1, and dont include NaNs
    assert np.all(tr['p_value'].values <= 1)
    assert np.all(tr['p_value'].values >= 0)
    assert np.all(~np.isnan(tr['p_value'].values)) 

    return tr

def get_p_val(trace, response_window, frame_rate):
    '''
        Computes a p-value for the trace by comparing the dff in the response_window to the same sized trace before the response_window. 
        PROBLEM: This should be computed by comparing to spontaneous activity to be consistent with the flash_response_df

        INPUTS:
        trace, the dff trace for this cell/trial 
        response_window, [start_time, end_time] the time in seconds from the start of trace to asses whether the activity is significant
        frame_rate, the number of samples in trace per second. 

        OUTPUTS:
        a p-value
    '''
    response_window_duration = response_window[1] - response_window[0]
    baseline_end = int(response_window[0] * frame_rate)
    baseline_start = int((response_window[0] - response_window_duration) * frame_rate)
    stim_start = int(response_window[0] * frame_rate)
    stim_end = int((response_window[0] + response_window_duration) * frame_rate)
    (_, p) = stats.f_oneway(trace[baseline_start:baseline_end], trace[stim_start:stim_end])
    return p

def annotate_trial_response_df_with_pref_stim(trial_response_df):
    '''
        Computes the preferred stimulus for each cell/trial combination. Preferred image is computed by seeing which image evoked the largest average mean_response across all change_images. 

        INPUTS:
        trial_response_df, the trial_response_df to be annotated

        OUTPUTS:
        a copy of trial_response_df with a new column appended 'pref_stim' which is a boolean TRUE/FALSE for whether that change_image was that cell's preferred image. 
       
        ASSERTS:
        Each cell has one unique preferred stimulus 
    '''
    
    # Copy the trial_response_df
    rdf = trial_response_df.copy()
    
    # Set up empty column
    rdf['pref_stim'] = False
    
    # get average mean_response for each cell X change_image
    mean_response = rdf.groupby(['cell_specimen_id', 'change_image_name']).apply(get_mean_sem_trace)
    m = mean_response.unstack()

    # set index to be cell/image pairs 
    rdf = rdf.reset_index()
    rdf = rdf.set_index(['cell_specimen_id','change_image_name'])

    # Iterate through cells, and determine which change_image evoked the largest response
    for cell in m.index:
        image_index = np.where(m.loc[cell]['mean_response'].values == np.max(m.loc[cell]['mean_response'].values))[0][0]
        pref_image = m.loc[cell]['mean_response'].index[image_index]
        
        # Update the cell X change_image pairs to have the pref_stim set to True
        rdf.at[(cell,pref_image),'pref_stim'] = True

    # Test to ensure preferred stimulus is unique for each cell
    for cell in rdf.reset_index()['cell_specimen_id'].unique(): 
        assert len(rdf.reset_index().set_index('cell_specimen_id').loc[cell].query('pref_stim').change_image_name.unique()) == 1

    # Reset index to be cell/trial pairs
    rdf = rdf.reset_index()
    rdf = rdf.set_index(['cell_specimen_id','trial_id'])
    return rdf

def get_mean_sem_trace(group):
    '''
        Computes the average and sem of the mean_response column

        INPUTS:
        group, a pandas group
        
        OUTPUT:
        a pandas series with the mean_response, sem_response, mean_trace, sem_trace, and mean_responses computed for the group. 
    '''
    mean_response = np.mean(group['mean_response'])
    mean_responses = group['mean_response'].values
    sem_response = np.std(group['mean_response'].values) / np.sqrt(len(group['mean_response'].values))
    mean_trace = np.mean(group['dff_trace'])
    sem_trace = np.std(group['dff_trace'].values) / np.sqrt(len(group['dff_trace'].values))
    return pd.Series({'mean_response': mean_response, 'sem_response': sem_response,
                      'mean_trace': mean_trace, 'sem_trace': sem_trace,
                      'mean_responses': mean_responses})

def get_trial_response_df(session, response_analysis_params):
    '''
        Computes the trial_response_df for the session
        PROBLEM: Ignores aborted trials    

        INPUTS:
        session, a behaviorOphysSession object to be analyzed
        response_analysis_params, a dictionary with keys:
            'window_around_timepoint_seconds'   The window around the change_time to use in the dff trace
            'response_window_duration_seconds'  The duration after the change time to use in the mean_response
            'baseline_window_duration_seconds'  The duration  before the change time to use as the baseline_response
    
        OUTPUTS:
        trial_response_df, a pandas dataframe with multi-index (cell_specimen_id/trial_id), and columns:
            cell_roi_id, this sessions roi_id
            mean_response, the average dff in the response_window
            baseline_response, the average dff in the baseline window
            dff_trace, the dff_trace in the window_around_timepoint_seconds
            dff_trace_timestamps, the timestamps for the dff_trace
    '''
    frame_rate = 31. #PROBLEM, shouldnt hard code this here

    # get data to analyze
    dff_traces = session.dff_traces.copy()
    trials = session.trials.copy()
    trials = trials[trials.aborted==False] # PROBLEM

    # get params to define response window, in seconds
    window_around_timepoint_seconds = response_analysis_params['window_around_timepoint_seconds']
    response_window_duration_seconds = response_analysis_params['response_window_duration_seconds']
    baseline_window_duration_seconds = response_analysis_params['baseline_window_duration_seconds']
    mean_response_window_seconds = [np.abs(window_around_timepoint_seconds[0]), 
                        np.abs(window_around_timepoint_seconds[0]) + response_window_duration_seconds]
    baseline_window_seconds = [np.abs(window_around_timepoint_seconds[0]) - baseline_window_duration_seconds, 
                        np.abs(window_around_timepoint_seconds[0])]

    # Set up multi-index dataframe
    cell_trial_combinations = itertools.product(dff_traces.index,trials.index)
    index = pd.MultiIndex.from_tuples(cell_trial_combinations, names=['cell_specimen_id', 'trial_id'])
    df = pd.DataFrame(index=index)

    # Iterate through cell/trial pairs, and construct the columns
    traces_list = []
    trace_timestamps_list = []
    for cell_specimen_id, trial_id in itertools.product(dff_traces.index,trials.index):
        timepoint = trials.loc[trial_id]['change_time']
        cell_roi_id = dff_traces.loc[cell_specimen_id]['cell_roi_id']
        full_cell_trace = dff_traces.loc[cell_specimen_id, 'dff']
        trace, trace_timestamps = get_trace_around_timepoint(full_cell_trace, timepoint, session.ophys_timestamps, 
                                                             window_around_timepoint_seconds, frame_rate) 
        mean_response = get_mean_in_window(trace, mean_response_window_seconds, frame_rate) 
        baseline_response = get_mean_in_window(trace, baseline_window_seconds, frame_rate) 

        traces_list.append(trace)
        trace_timestamps_list.append(trace_timestamps)
        df.loc[(cell_specimen_id, trial_id), 'cell_roi_id'] = int(cell_roi_id)
        df.loc[(cell_specimen_id, trial_id), 'mean_response'] = mean_response
        df.loc[(cell_specimen_id, trial_id), 'baseline_response'] = baseline_response
    df.insert(loc=1, column='dff_trace', value=traces_list)
    df.insert(loc=2, column='dff_trace_timestamps', value=trace_timestamps_list)
    return df


if __name__=='__main__':
    # Load cache 
    cache_json = {'manifest_path': '/allen/programs/braintv/workgroups/nc-ophys/visual_behavior/SWDB_2019/visual_behavior_data_manifest.csv',
                  'nwb_base_dir': '/allen/programs/braintv/workgroups/nc-ophys/visual_behavior/SWDB_2019/nwb_files',
                  'analysis_files_base_dir': '/allen/programs/braintv/workgroups/nc-ophys/visual_behavior/SWDB_2019/extra_files_final'
                  }
    cache = bpc.BehaviorProjectCache(cache_json)

    case=0
    if case == 0:
        # this is the main use case
        experiment_id = sys.argv[1] # get experiment_id to analyze

        # Load session object
        # experiment_id = cache.manifest.iloc[5]['ophys_experiment_id']
        # nwb_path = cache.get_nwb_filepath(experiment_id)
        # api = BehaviorOphysNwbApi(nwb_path, filter_invalid_rois=True)
        # session = BehaviorOphysSession(api)

        # Get the session using the cache so that the change time fix is applied
        session = cache.get_session(experiment_id)
        change_times = session.trials['change_time'][~pd.isnull(session.trials['change_time'])].values
        flash_times = session.stimulus_presentations['start_time'].values
        assert np.all(np.isin(change_times, flash_times))

        output_path = '/allen/programs/braintv/workgroups/nc-ophys/visual_behavior/SWDB_2019/extra_files_final'

        response_analysis_params = {'window_around_timepoint_seconds':[-4,8],
                                   'response_window_duration_seconds': 0.5,
                                   'baseline_window_duration_seconds': 0.5}

        trial_response_df = get_trial_response_df(session, response_analysis_params)

        trial_metadata = session.trials.copy()
        trial_metadata.index.names = ['trial_id']
        trial_response_df = trial_response_df.join(trial_metadata)
        trial_response_df = add_p_vals_tr(trial_response_df)
        trial_response_df = annotate_trial_response_df_with_pref_stim(trial_response_df)

        output_fn = os.path.join(output_path, 'trial_response_df_{}.h5'.format(experiment_id))
        print('Writing trial response df to {}'.format(output_fn))
        trial_response_df.to_hdf(output_fn, key='df', complib='bzip2', complevel=9)

    elif case == 1:
        # This is a debugging case 
        experiment_id  = 846487947

        # api = BehaviorOphysLimsApi(experiment_id)
        # session = BehaviorOphysSession(api)
        # nwb_path = cache.get_nwb_filepath(experiment_id)
        # api = BehaviorOphysNwbApi(nwb_path)
        # session = BehaviorOphysSession(api)

        session = cache.get_session(experiment_id)

        change_times = session.trials['change_time'][~pd.isnull(session.trials['change_time'])].values
        flash_times = session.stimulus_presentations['start_time'].values
        assert np.all(np.isin(change_times, flash_times))

        output_path = '/allen/programs/braintv/workgroups/nc-ophys/visual_behavior/SWDB_2019/extra_files_final'

        response_analysis_params = {'window_around_timepoint_seconds':[-4,8],
                                   'response_window_duration_seconds': 0.5,
                                   'baseline_window_duration_seconds': 0.5}

        trial_response_df = get_trial_response_df(session, response_analysis_params)

        trial_metadata = session.trials.copy()
        trial_metadata.index.names = ['trial_id']
        trial_response_df = trial_response_df.join(trial_metadata)
        trial_response_df = add_p_vals_tr(trial_response_df)
        trial_response_df = annotate_trial_response_df_with_pref_stim(trial_response_df)

==> allensdk-2.8.0-py3-none-any.whl: allensdk/brain_observatory/behavior/swdb/save_trial_response_df.py <==
import sys
import os
import numpy as np
import pandas as pd
from scipy import stats
import itertools

from allensdk.brain_observatory.behavior.behavior_ophys_session import BehaviorOphysSession
from allensdk.brain_observatory.behavior.session_apis.data_io import (
    BehaviorOphysNwbApi, BehaviorOphysLimsApi)
from allensdk.brain_observatory.behavior.swdb import behavior_project_cache as bpc
from importlib import reload; reload(bpc)
from allensdk.brain_observatory.behavior.swdb.analysis_tools import get_nearest_frame, get_trace_around_timepoint, get_mean_in_window

'''
    This file contains functions and a script for computing the trial_response_df dataframe.
    This file was hastily constructed before friday harbor. Places where there are known issues are flagged with PROBLEM
'''


def add_p_vals_tr(tr,response_window = [4,4.5]):
    '''
        Computes the p value for each cell's response on each trial. The p-value is computed using the function 'get_p_val' 

        INPUT:
        tr, trial_response_dataframe
        response_window, the time points in the dff trace to use for computing the p-value. 
                        PROBLEM: The default value here assumes that the dff_trace starts 4 seconds before the change time. 
                        This should be set up with more care and flexibility. 

        OUTPUTS:
        tr, the same trial_response_dataframe, with a new column 'p_value' appended. 
    
        ASSERTS:
        tr['p_value'] is inclusively bounded between 0 and 1, and does not include NaNs
    '''
    
    # Set up empty column
    tr['p_value'] = 1.
    ophys_frame_rate=31. #Shouldn't hard code this PROBLEM
    
    # Iterate over trial/cell pairs, and compute p-value
    for index,row in tr.iterrows():
        tr.at[index,'p_value'] = get_p_val(row.dff_trace, response_window, ophys_frame_rate)

    # Test to ensure p values are bounded between 0 and 1, and dont include NaNs
    assert np.all(tr['p_value'].values <= 1)
    assert np.all(tr['p_value'].values >= 0)
    assert np.all(~np.isnan(tr['p_value'].values)) 

    return tr

def get_p_val(trace, response_window, frame_rate):
    '''
        Computes a p-value for the trace by comparing the dff in the response_window to the same sized trace before the response_window. 
        PROBLEM: This should be computed by comparing to spontaneous activity to be consistent with the flash_response_df

        INPUTS:
        trace, the dff trace for this cell/trial 
        response_window, [start_time, end_time] the time in seconds from the start of trace to asses whether the activity is significant
        frame_rate, the number of samples in trace per second. 

        OUTPUTS:
        a p-value
    '''
    response_window_duration = response_window[1] - response_window[0]
    baseline_end = int(response_window[0] * frame_rate)
    baseline_start = int((response_window[0] - response_window_duration) * frame_rate)
    stim_start = int(response_window[0] * frame_rate)
    stim_end = int((response_window[0] + response_window_duration) * frame_rate)
    (_, p) = stats.f_oneway(trace[baseline_start:baseline_end], trace[stim_start:stim_end])
    return p

def annotate_trial_response_df_with_pref_stim(trial_response_df):
    '''
        Computes the preferred stimulus for each cell/trial combination. Preferred image is computed by seeing which image evoked the largest average mean_response across all change_images. 

        INPUTS:
        trial_response_df, the trial_response_df to be annotated

        OUTPUTS:
        a copy of trial_response_df with a new column appended 'pref_stim' which is a boolean TRUE/FALSE for whether that change_image was that cell's preferred image. 
       
        ASSERTS:
        Each cell has one unique preferred stimulus 
    '''
    
    # Copy the trial_response_df
    rdf = trial_response_df.copy()
    
    # Set up empty column
    rdf['pref_stim'] = False
    
    # get average mean_response for each cell X change_image
    mean_response = rdf.groupby(['cell_specimen_id', 'change_image_name']).apply(get_mean_sem_trace)
    m = mean_response.unstack()

    # set index to be cell/image pairs 
    rdf = rdf.reset_index()
    rdf = rdf.set_index(['cell_specimen_id','change_image_name'])

    # Iterate through cells, and determine which change_image evoked the largest response
    for cell in m.index:
        image_index = np.where(m.loc[cell]['mean_response'].values == np.max(m.loc[cell]['mean_response'].values))[0][0]
        pref_image = m.loc[cell]['mean_response'].index[image_index]
        
        # Update the cell X change_image pairs to have the pref_stim set to True
        rdf.at[(cell,pref_image),'pref_stim'] = True

    # Test to ensure preferred stimulus is unique for each cell
    for cell in rdf.reset_index()['cell_specimen_id'].unique(): 
        assert len(rdf.reset_index().set_index('cell_specimen_id').loc[cell].query('pref_stim').change_image_name.unique()) == 1

    # Reset index to be cell/trial pairs
    rdf = rdf.reset_index()
    rdf = rdf.set_index(['cell_specimen_id','trial_id'])
    return rdf

def get_mean_sem_trace(group):
    '''
        Computes the average and sem of the mean_response column

        INPUTS:
        group, a pandas group
        
        OUTPUT:
        a pandas series with the mean_response, sem_response, mean_trace, sem_trace, and mean_responses computed for the group. 
    '''
    mean_response = np.mean(group['mean_response'])
    mean_responses = group['mean_response'].values
    sem_response = np.std(group['mean_response'].values) / np.sqrt(len(group['mean_response'].values))
    mean_trace = np.mean(group['dff_trace'])
    sem_trace = np.std(group['dff_trace'].values) / np.sqrt(len(group['dff_trace'].values))
    return pd.Series({'mean_response': mean_response, 'sem_response': sem_response,
                      'mean_trace': mean_trace, 'sem_trace': sem_trace,
                      'mean_responses': mean_responses})

def get_trial_response_df(session, response_analysis_params):
    '''
        Computes the trial_response_df for the session
        PROBLEM: Ignores aborted trials    

        INPUTS:
        session, a behaviorOphysSession object to be analyzed
        response_analysis_params, a dictionary with keys:
            'window_around_timepoint_seconds'   The window around the change_time to use in the dff trace
            'response_window_duration_seconds'  The duration after the change time to use in the mean_response
            'baseline_window_duration_seconds'  The duration  before the change time to use as the baseline_response
    
        OUTPUTS:
        trial_response_df, a pandas dataframe with multi-index (cell_specimen_id/trial_id), and columns:
            cell_roi_id, this sessions roi_id
            mean_response, the average dff in the response_window
            baseline_response, the average dff in the baseline window
            dff_trace, the dff_trace in the window_around_timepoint_seconds
            dff_trace_timestamps, the timestamps for the dff_trace
    '''
    frame_rate = 31. #PROBLEM, shouldnt hard code this here

    # get data to analyze
    dff_traces = session.dff_traces.copy()
    trials = session.trials.copy()
    trials = trials[trials.aborted==False] # PROBLEM

    # get params to define response window, in seconds
    window_around_timepoint_seconds = response_analysis_params['window_around_timepoint_seconds']
    response_window_duration_seconds = response_analysis_params['response_window_duration_seconds']
    baseline_window_duration_seconds = response_analysis_params['baseline_window_duration_seconds']
    mean_response_window_seconds = [np.abs(window_around_timepoint_seconds[0]), 
                        np.abs(window_around_timepoint_seconds[0]) + response_window_duration_seconds]
    baseline_window_seconds = [np.abs(window_around_timepoint_seconds[0]) - baseline_window_duration_seconds, 
                        np.abs(window_around_timepoint_seconds[0])]

    # Set up multi-index dataframe
    cell_trial_combinations = itertools.product(dff_traces.index,trials.index)
    index = pd.MultiIndex.from_tuples(cell_trial_combinations, names=['cell_specimen_id', 'trial_id'])
    df = pd.DataFrame(index=index)

    # Iterate through cell/trial pairs, and construct the columns
    traces_list = []
    trace_timestamps_list = []
    for cell_specimen_id, trial_id in itertools.product(dff_traces.index,trials.index):
        timepoint = trials.loc[trial_id]['change_time']
        cell_roi_id = dff_traces.loc[cell_specimen_id]['cell_roi_id']
        full_cell_trace = dff_traces.loc[cell_specimen_id, 'dff']
        trace, trace_timestamps = get_trace_around_timepoint(full_cell_trace, timepoint, session.ophys_timestamps, 
                                                             window_around_timepoint_seconds, frame_rate) 
        mean_response = get_mean_in_window(trace, mean_response_window_seconds, frame_rate) 
        baseline_response = get_mean_in_window(trace, baseline_window_seconds, frame_rate) 

        traces_list.append(trace)
        trace_timestamps_list.append(trace_timestamps)
        df.loc[(cell_specimen_id, trial_id), 'cell_roi_id'] = int(cell_roi_id)
        df.loc[(cell_specimen_id, trial_id), 'mean_response'] = mean_response
        df.loc[(cell_specimen_id, trial_id), 'baseline_response'] = baseline_response
    df.insert(loc=1, column='dff_trace', value=traces_list)
    df.insert(loc=2, column='dff_trace_timestamps', value=trace_timestamps_list)
    return df


if __name__=='__main__':
    # Load cache 
    cache_json = {'manifest_path': '/allen/programs/braintv/workgroups/nc-ophys/visual_behavior/SWDB_2019/visual_behavior_data_manifest.csv',
                  'nwb_base_dir': '/allen/programs/braintv/workgroups/nc-ophys/visual_behavior/SWDB_2019/nwb_files',
                  'analysis_files_base_dir': '/allen/programs/braintv/workgroups/nc-ophys/visual_behavior/SWDB_2019/extra_files_final'
                  }
    cache = bpc.BehaviorProjectCache(cache_json)

    case=0
    if case == 0:
        # this is the main use case
        experiment_id = sys.argv[1] # get experiment_id to analyze

        # Load session object
        # experiment_id = cache.manifest.iloc[5]['ophys_experiment_id']
        # nwb_path = cache.get_nwb_filepath(experiment_id)
        # api = BehaviorOphysNwbApi(nwb_path, filter_invalid_rois=True)
        # session = BehaviorOphysSession(api)

        # Get the session using the cache so that the change time fix is applied
        session = cache.get_session(experiment_id)
        change_times = session.trials['change_time'][~pd.isnull(session.trials['change_time'])].values
        flash_times = session.stimulus_presentations['start_time'].values
        assert np.all(np.isin(change_times, flash_times))

        output_path = '/allen/programs/braintv/workgroups/nc-ophys/visual_behavior/SWDB_2019/extra_files_final'

        response_analysis_params = {'window_around_timepoint_seconds':[-4,8],
                                   'response_window_duration_seconds': 0.5,
                                   'baseline_window_duration_seconds': 0.5}

        trial_response_df = get_trial_response_df(session, response_analysis_params)

        trial_metadata = session.trials.copy()
        trial_metadata.index.names = ['trial_id']
        trial_response_df = trial_response_df.join(trial_metadata)
        trial_response_df = add_p_vals_tr(trial_response_df)
        trial_response_df = annotate_trial_response_df_with_pref_stim(trial_response_df)

        output_fn = os.path.join(output_path, 'trial_response_df_{}.h5'.format(experiment_id))
        print('Writing trial response df to {}'.format(output_fn))
        trial_response_df.to_hdf(output_fn, key='df', complib='bzip2', complevel=9)

    elif case == 1:
        # This is a debugging case 
        experiment_id  = 846487947

        # api = BehaviorOphysLimsApi(experiment_id)
        # session = BehaviorOphysSession(api)
        # nwb_path = cache.get_nwb_filepath(experiment_id)
        # api = BehaviorOphysNwbApi(nwb_path)
        # session = BehaviorOphysSession(api)

        session = cache.get_session(experiment_id)

        change_times = session.trials['change_time'][~pd.isnull(session.trials['change_time'])].values
        flash_times = session.stimulus_presentations['start_time'].values
        assert np.all(np.isin(change_times, flash_times))

        output_path = '/allen/programs/braintv/workgroups/nc-ophys/visual_behavior/SWDB_2019/extra_files_final'

        response_analysis_params = {'window_around_timepoint_seconds':[-4,8],
                                   'response_window_duration_seconds': 0.5,
                                   'baseline_window_duration_seconds': 0.5}

        trial_response_df = get_trial_response_df(session, response_analysis_params)

        trial_metadata = session.trials.copy()
        trial_metadata.index.names = ['trial_id']
        trial_response_df = trial_response_df.join(trial_metadata)
        trial_response_df = add_p_vals_tr(trial_response_df)
        trial_response_df = annotate_trial_response_df_with_pref_stim(trial_response_df)
